﻿/********************************************************************************/
/* 	gamecore.d																	*/
/*------------------------------------------------------------------------------*/
/*	製作		D.K（あるふぁ～秘密基地）										*/
/*	製作開始	2008/05/22														*/
/*	MAIL		dk@red.interq.or.jp												*/
/*	URL			http://maglog.jp/alpha-secret-base/								*/
/*																				*/
/*	このソースは「やわらかライセンス」の元で配布されています。					*/
/*-更新履歴---------------------------------------------------------------------*/
/*	2008/05/22	初版															*/
/*-その他-----------------------------------------------------------------------*/
/*	なし																		*/
/********************************************************************************/

private import state;
private import hell2;
private import boot;
private import actor;
private import vector;
private import keypad;
private import clickable;
//private import configparser;

import config;

//private import std.math;

/********************************************************************************/

//ConfigParser configparser = null;
const static string CONFIG_PATH = "config.ini";

alias ubyte	Uint8;
alias byte	Sint8;
alias ushort	Uint16;
alias short	Sint16;
alias uint	Uint32;
alias int		Sint32;

/* GameMain *********************************************************************/
/*	ゲーム本体の状態															*/
/********************************************************************************/

class GameMain : GameState
{
public:
	//-----------------------------------------------------------------------------
	//システム制御用
	//-----------------------------------------------------------------------------
	enum SCENE{
		INIT = 0,
		START,
		MAIN,
		CLEAR,
		END,
		PAUSE,
		RELESE,
		
		//以下ゲーム内の演出
		PLAYER_WARP,
		ENEMY_GETWAIT,
		PLAYER_MISSWAIT,

		EXIT,
	};
	

	ConfigParser savedata;			// セーブデータ

	int timer;						// 汎用タイマー
	SCENE iSceneStatus;				//画面の状態
	int stage;						// ステージ
	int stage_start_count;			// ステージ開始処理
	int stage_clear_count;			// ステージ開始処理
	int game_score = 0;				//スコア
	int hi_score = 0;				//ハイスコア
	int game_time_count = 0;		//ゲーム終了までの残り時間
	int menu_input = 0;				//敵を倒したときのWAITカウンタ

	int replaymode;					//リプレイモード
	float pos_x = 20;					//位置
	float pos_y = 20;					//位置
	float pos_addx = 0;				//加速度
	float pos_addy = 0;				//加速度
	int enelgy = 1000;				//エネルギー
	int se_wait = 0;				//噴射音
	int is_up = 0;					//上昇しているか？
	int muteki = 0;					//無敵カウント

	Uint32 mapcolor = 0;			//マップの色
		
	Keypad keypad;					// キーボード/ゲームパッド取得用
	//-----------------------------------------------------------------------------
	//ゲーム別に使用される特別処理用変数
	//-----------------------------------------------------------------------------

	//-----------------------------------------------------------------------------
	//	コンストラクタ
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	this()
	{
		setMsg( GameState.MSG_NONE );
		// 各種初期化
		timer = 0;

		savedata          = new ConfigParser();
		keypad = new Keypad();
		keypad.SetUseCancel( false );

		iSceneStatus = SCENE.INIT;

	}

	
	//-----------------------------------------------------------------------------
	//	シーンの初期化
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void init( )
	{
Hell_write( "Gamecoer init\n" );
		//リソースの読み込み
		Hell_loadTexture("tama","resource/image/tama.bmp" , [0,255,0]);
		Hell_loadTexture("map" ,"resource/image/map01.bmp" , [0,255,0]);
		Hell_loadTexture("menu","resource/image/menu.bmp" , [0,255,0]);
		Hell_loadTexture("rs_01","resource/image/rs_01.bmp" , [0,255,0]);
		Hell_loadTexture("rs_02","resource/image/rs_02.bmp" , [0,255,0]);

		Hell_loadWAV("syu",         "resource/sound/syu.wav");
		Hell_loadWAV("ban",         "resource/sound/ban.wav");
		Hell_loadWAV("pon",         "resource/sound/pon.wav");
		Hell_loadWAV("atk1",         "resource/sound/atk1.wav");

		//セーブデータ
		try
		{
			savedata.load(CONFIG_PATH);
		}
		catch(Exception e)
		{
			savedata.set("HI_SCORE",         999999);
			savedata.set("HOSHI",         1);
			savedata.write();
		}
		hi_score = savedata.getint("HI_SCORE");

		game_time_count = 0;		//ゲーム終了までの残り時間

Hell_write( "Gamecoer init\n" );

		iSceneStatus = SCENE.START;

		replaymode = Hell_GetGameflag( 0 );	//リプレイモード
		if ( replaymode != 0 )
		{
			//リプレイを再生する
			try
			{
				keypad.load( "replay/replay_01.bin" );
			}
			catch(Exception e)
			{
				//ファイルのオープンに失敗した
				iSceneStatus = SCENE.EXIT;
			}
		}

	}
	


	//-----------------------------------------------------------------------------
	//	シーンの後処理
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void relese( )
	{
		iSceneStatus = SCENE.EXIT;
	}
	//-----------------------------------------------------------------------------
	//	開始演出
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void SeneStart_move( )
	{
		int i;

		if ( stage_start_count < ( 60 * 3 ) )
		{
			stage_start_count++;
		}
		else 
		{
			Hell_playBgm( "resource/sound/bgm.ogg", -1, 0 );
			iSceneStatus = SCENE.MAIN;
			stage_start_count = 0;
		}
	}
	//-----------------------------------------------------------------------------
	//	ステージ開始表示処理
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void SeneStart_draw( )
	{
		if ( stage_start_count > ( 60 * 2 ) )
		{
			Hell_drawFont(" GO! " , 240, 200 , 4 , 255,255,255);
		}
		else
		{
			Hell_drawFont("READY" , 240, 200 , 4 , 255,255,255);
		}
	}
	//-----------------------------------------------------------------------------
	//	クリア演出の処理
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void SeneClear_move( )
	{
		if ( stage_clear_count > ( 60 * 2 ) )
		{
			if( keypad.isPushEnter(true) )
			{
				iSceneStatus = SCENE.RELESE;
			}
		}
		else 
		{
			stage_clear_count++;
		}
		
		
	}
	//-----------------------------------------------------------------------------
	//	クリア演出の表示
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void SeneClear_draw( )
	{
		int min;
		int sec;
		int msec;
		char[] str;


		Hell_drawTexture("rs_01"
			, 0
			, 0
			, 0
			, 0
			, 640, 480
			, 1.0 , 1.0
			, 0
			,255, 255, 255, 255 );

		Hell_setAlpha(HELL_ALPHA_NORMAL);
//		Hell_drawFont( "HI SCORE" , 400, 10 , 2 , 255,255,255);
//		Hell_drawFont( std.string.toString( hi_score ) , 400, 30 , 4 , 255,255,255);

		Hell_drawFont( "RESULTS" , 200, 250 , 4 , 255,255,255);

		min = ( game_score / 60 ) / 60;
		sec = ( game_score / 60 ) % 60;
		msec = game_score % 60;
		str = std.string.format( "%02d:%02d:%02d", min, sec, msec );

		Hell_drawFont( "TIME" ,    210, 300 , 2 , 255,255,255);
		Hell_drawFont( str , 220, 320 , 4 , 255,255,255);
		if ( hi_score >= game_score )
		{
			Hell_drawFont( "NEW RECORD !",170, 390 , 4 , 255,255,255);
		}

		if ( stage_clear_count > ( 60 * 2 ) )
		{
			Hell_drawFont("PRESS Z OR 1 BUTTON TO EXIT" , 100, 440 , 2 , 255,255,255);
		}
		else
		{
			Hell_drawFont(" SAITA !" , 200, 120 , 4 , 255,255,255);
		}
	}
	//-----------------------------------------------------------------------------
	//	ゲームオーバー演出
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void SeneEnd_move( )
	{
		if ( stage_clear_count > ( 60 * 2 ) )
		{
			if( keypad.isPushEnter(true) )
			{
				iSceneStatus = SCENE.RELESE;
			}
		}
		else 
		{
			stage_clear_count++;
		}
	}
	//-----------------------------------------------------------------------------
	//	ゲームオーバー演出の表示
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void SeneEnd_draw( )
	{
		Hell_drawTexture("rs_02"
			, 0
			, 0
			, 0
			, 0
			, 640, 480
			, 1.0 , 1.0
			, 0
			,255, 255, 255, 255 );

		if ( stage_clear_count > ( 60 * 2 ) )
		{
			Hell_drawFont("PRESS Z OR 1 BUTTON TO EXIT" , 100, 440 , 2 , 255,255,255);
		}
		else
		{
			Hell_drawFont(" KARETA !" , 200, 220 , 4 , 255,255,255);
		}
	}
	
	//-----------------------------------------------------------------------------
	//	オブジェクトの動作
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void move()
	{
		
		switch( iSceneStatus )
		{
		case SCENE.INIT:
			//初期化
			init( );
			break;
		case SCENE.START:
			//開始
			SeneStart_move( );
			break;
		case SCENE.MAIN:
			keypad.update( );	//リプレイキー入力の更新
			//メイン
			system_move( );

			break;
		case SCENE.CLEAR:
			//クリア
//			effect_move( );
//			mob_move( );
			SeneClear_move( );
			break;
		case SCENE.END:
			//ゲームオーバー
			SeneEnd_move( );
			break;
		case SCENE.PAUSE:
			//一時停止
			pause_move( );
			break;
		case SCENE.RELESE:
			//後処理
			relese( );
			break;

		case SCENE.EXIT:
		default:
			//シーンの終了
			setMsg(GameState.MSG_REQ_TITLE);
			break;
		}
	}
	
	//-----------------------------------------------------------------------------
	//	オブジェクトの描画
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void draw()
	{
		// 作画開始
		Hell_begin();
		resetPerspective();
		Hell_setAlpha(HELL_ALPHA_NORMAL);
		
		switch( iSceneStatus )
		{
		case SCENE.INIT:
			//初期化
			break;
		case SCENE.START:
			//開始
			system_draw( );

			SeneStart_draw( );
			break;
		case SCENE.MAIN:
			//メイン
			system_draw( );
			break;
		case SCENE.CLEAR:
			//クリア
			system_draw( );
			SeneClear_draw( );
			break;
		case SCENE.END:
			//ゲームーバー
			system_draw( );
			SeneEnd_draw( );
			break;
		case SCENE.PAUSE:
			//一時停止
			system_draw( );
			pause_draw( );
			break;
		case SCENE.RELESE:
			//後処理
			break;
			
			
		case SCENE.EXIT:
		default:
			break;
		}
// for debug code start
//Hell_drawFPS( 550, 10 );
//Hell_drawFont( std.string.toString( cast( Uint32 )mapcolor ), 10, 10 * 1, 1 , 255,255,255 );
//Hell_drawFont( std.string.toString( cast( int )pos_addx ), 10, 10 * 2, 1 , 255,255,255 );
//Hell_drawFont( std.string.toString( cast( int )pos_addy ), 10, 10 * 3, 1 , 255,255,255 );
//Hell_drawFont( std.string.toString( cast( int )motion ), 10, 10 * 4, 1 , 255,255,255 );
//Hell_drawFont( std.string.toString( cast( int )debug_disp[0] ), 10, 10 * 5, 1 , 255,255,255 );
//Hell_drawFont( std.string.toString( cast( int )jump_fream_count ), 10, 10 * 6, 1 , 255,255,255 );
// for debug code end

	}
	
	// 画面サイズ取得
	int getScreenWidth(){return boot.g_width;}
	int getScreenHeight(){return boot.g_height;}
	
	string zfill(string str , int n)
	{
		for(int t=str.length;t<n;t++){
			str = "0" ~ str;
		}
		return str;
	}
	
	//-----------------------------------------------------------------------------
	//	ゲーム情報、システムの処理
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void system_move( )
	{
		//残り時間のカウント
		game_time_count++;



		if(keypad.isPressLeft( ) )
		{
			pos_addx -= 20.0f;					//進んだ距離
		}
		if(keypad.isPressRight( ) )
		{
			pos_addx += 20.0f;					//進んだ距離
		}
		
		pos_addx = cast( int )( cast( float )pos_addx * cast( float )0.98f );
		
		pos_x += ( pos_addx / 100.0f );
		if ( pos_x > 640.0f )
		{
			pos_x -= 640.0f;
		}
		if ( pos_x < 0 )
		{
			pos_x += 640.0f;
		}

		is_up = 0;
		if(keypad.isPressEnter( ) )
		{
			enelgy -= 1;
			if ( enelgy > 0 )
			{
				pos_addy -= 20.0f; 
				se_wait++;
				is_up = 1;
			}
		}
		else
		{
			se_wait = 0;
		}
		if ( ( se_wait % 30 ) == 1 )
		{
			Hell_playWAV( "syu", 0, -1 );
		}
		if ( enelgy < 0 )
		{
			enelgy = 0;
		}
		
		pos_addy += 10.0f;

		pos_y += cast( int )( pos_addy / 98.0f );
/*

		if(keypad.isPressUp(true))
		{
			pos_addy -= 20;					//進んだ距離
		}
		if(keypad.isPressDown(true))
		{
			pos_addy += 20;					//進んだ距離
		}
		pos_addy = cast( int )( cast( float )pos_addy * cast( float )0.98 );
		pos_y += cast( int )( pos_addy / 80 );
*/

		//地面の色を取得
		{
			int x = 0;
			int y = 0;
			mapcolor = 0;
			for ( x = 4; x < 12; x++ )
			{
				for ( y = 4; y < 12; y++ )
				{
					if (
					     ( ( cast( int )( pos_x - 8 + x ) < 640 ) && ( cast( int )( pos_x - 8 + x ) > 0 ) )
					  && ( ( cast( int )( pos_y - 8 + y ) < 480 ) && ( cast( int )( pos_y - 8 + y ) > 0 ) )
					   )
					{
						Uint32 mc = GetPixel( "map", cast( int )( pos_x - 8 + x ), cast( int )( pos_y - 8 + y ) );
						if ( mapcolor == 0 )
						{
							if ( mapcolor == 0xff0000ff )
							{
							}
							else
							{
								mapcolor = mc;
							}
						}
					}
				}
			}
		}
		
		//白に触った
		if ( mapcolor == 0xffffffff )
		{
			if ( muteki == 0 )
			{
				if (
				     ( ( pos_addx > 400.0f ) || ( pos_addx < -400.0f ) )
				  || ( ( pos_addy > 400.0f ) || ( pos_addy < -400.0f ) )
				   )
				{
					Hell_stopBgm( 1000 );
					Hell_playWAV( "ban", 0, -1 );
					iSceneStatus = SCENE.END;
				}
				else
				{
					Hell_playWAV( "atk1", 0, -1 );
					pos_addx = pos_addx * -2;					//進んだ距離
					pos_addy = pos_addy * -2;					//進んだ距離
					muteki = 3;
				}
			}
		}

		//青に触った
		if ( mapcolor == 0xff0000ff )
		{
			if (
			     ( ( pos_addx > 400.0f ) || ( pos_addx < -400.0f ) )
			  || ( ( pos_addy > 400.0f ) || ( pos_addy < -400.0f ) )
			   )
			{
				if ( muteki == 0 )
				{
					Hell_stopBgm( 1000 );
					Hell_playWAV( "ban", 0, -1 );
					iSceneStatus = SCENE.END;
				}
			}
			else
			{
				if (
				     ( ( pos_addy < 150.0f ) && ( pos_addy > -150.0f ) )
				   )
				{
					Hell_playWAV( "pon", 0, -1 );
					Hell_playBgm( "resource/sound/results.ogg", 1, 0 );

					//ハイスコアの保存
					game_score = game_time_count;
					if ( replaymode == 0 )
					{
						if ( hi_score > game_score )
						{
							keypad.save( "replay/replay_01.bin" );	//リプレイの保存
							hi_score = game_score;
							savedata.set("HI_SCORE",   cast(int)hi_score);
							savedata.write();
						}
					}

					iSceneStatus = SCENE.CLEAR;

				}
				else
				{
					if ( muteki == 0 )
					{
						Hell_playWAV( "atk1", 0, -1 );
						pos_addx = pos_addx * -2;					//進んだ距離
						pos_addy = pos_addy * -2;					//進んだ距離
						muteki = 3;
					}
				}
			}
		}
		


		if ( ( pos_y >= 700.0f ) || ( pos_y < -100.0f ) )
		{
			Hell_stopBgm( 1000 );
			Hell_playWAV( "ban", 0, -1 );
			iSceneStatus = SCENE.END;
		}
		
		muteki--;
		if ( muteki < 0 )
		{
			muteki = 0;
		}
/*
		{
			Hell_playBgm( "resource/sound/results.ogg", 1, 0 );

			//ハイスコアの保存
			game_score = game_time_count;
			if ( hi_score > game_score )
			{
				keypad.save( "replay/replay_01.bin" );	//リプレイの保存
				hi_score = game_score;
				savedata.set("HI_SCORE",   cast(int)hi_score);
				savedata.write();
			}

			iSceneStatus = SCENE.CLEAR;
		}
*/		
		if ( iSceneStatus == SCENE.MAIN )
		{
			if(keypad.isPushCancel(true))
			{
				//ポーズ
//				iSceneStatus = SCENE.PAUSE;
//				menu_input = 0;
			}
		}
	}
	//-----------------------------------------------------------------------------
	//	ゲーム情報、システムの描画
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void system_draw( )
	{
		int min;
		int sec;
		int msec;
		char[] str;
		int r = 255;
		int g = 255;
		int b = 255;
		
		
		Hell_drawTexture("map"
			, 0
			, 0
			, 0
			, 0
			, 640, 480
			, 1.0 , 1.0
			, 0
			,255, 255, 255, 255 );
		
		if ( iSceneStatus != SCENE.END )
		{
			Hell_drawTexture("tama"
				, cast( int )pos_x - 8
				, cast( int )pos_y - 8
				, is_up * 16
				, 0
				, 16, 32
				, 1.0 , 1.0
				, 0
				,255, 255, 255, 255 );

			Hell_drawTexture("tama"
				, cast( int )pos_x + 640 - 8
				, cast( int )pos_y - 8
				, is_up * 16
				, 0
				, 16, 32
				, 1.0 , 1.0
				, 0
				,255, 255, 255, 255 );

			Hell_drawTexture("tama"
				, cast( int )pos_x - 640 - 8
				, cast( int )pos_y - 8
				, is_up * 16
				, 0
				, 16, 32
				, 1.0 , 1.0
				, 0
				,255, 255, 255, 255 );
		}
		
		
		Hell_drawFont( "POWER" , 420, 430 , 2 , 255,255,255);
		Hell_drawRect( 420 - 2, 450 - 2, 200 + 4, 10 + 4, 0, 0, 255, 255, 255, 255);
		Hell_drawRect( 420, 450, 200, 10, 0, 0, 0, 0, 0, 255);
		if ( enelgy > 0 )
		{
			Hell_drawRect( 420, 450, cast( int )( enelgy / 5 ), 10, 0, 0, 0, 255, 0, 255);
		} 
		
		
		
		
		
		
		
		min = ( game_time_count / 60 ) / 60;
		sec = ( game_time_count / 60 ) % 60;
		msec = game_time_count % 60;
		
		str = std.string.format( "TIME %02d:%02d:%02d", min, sec, msec );
		Hell_setAlpha(HELL_ALPHA_NORMAL);
		Hell_drawFont( str , 350, 4 , 2 , r,g,b);

		if ( replaymode != 0 )
		{
			Hell_drawFont( "REPLAY" , 4, 4 , 2 , r,g,b);
		}

/*
//		Hell_drawFont( "SCORE" , 10, 10 , 2 , 255,255,255);
//		Hell_drawFont( std.string.toString( game_score ) , 10, 30 , 4 , 255,255,255);

		min = ( hi_score / 60 ) / 60;
		sec = ( hi_score / 60 ) % 60;
		msec = hi_score % 60;

		Hell_drawFont( "HI SCORE" , 300, 10 , 2 , 255,255,255);
		str = std.string.format( "%02d:%02d:%02d", min, sec, msec );
		Hell_drawFont( str , 300, 30 , 4 , 255,255,255);
*/	
	}
	
	//-----------------------------------------------------------------------------
	//	ポーズの処理
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void pause_move( )
	{
		if(keypad.isPushUp(true))
		{
			Hell_playWAV( "select", 0, -1 );
			menu_input++;
			if ( menu_input > 1 )
			{
				menu_input = 0;
			}
		}
		else if(keypad.isPushDown(true))
		{
			Hell_playWAV( "select", 0, -1 );
			menu_input--;
			if ( menu_input < 0 )
			{
				menu_input = 1;
			}
		}
		if(keypad.isPushEnter(true))
		{
			switch ( menu_input )
			{
			case 0:
				//ポーズ解除
				iSceneStatus = SCENE.MAIN;
				break;
			case 1:
				//リトライ
				iSceneStatus = SCENE.RELESE;
				break;
			default:
				break;
			}
		}
		if(keypad.isPushCancel(true))
		{
			//ポーズ解除
			iSceneStatus = SCENE.MAIN;
		}
	}
	//-----------------------------------------------------------------------------
	//	ポーズの描画
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void pause_draw( )
	{
		Hell_setAlpha(HELL_ALPHA_NORMAL);
		Hell_drawTexture("menu"
			, 96 + 160
			, 64 + 120 + ( menu_input * 32 )
			, 320
			, 0
			, 32, 32
			, 1.0 , 1.0
			, 0
			,255, 255, 255, 255 );
		Hell_drawTexture("menu"
			, 160
			, 120
			, 0
			, 0
			, 320, 240
			, 1.0 , 1.0
			, 0
			,255, 255, 255, 255 );
	}
	
	
}


